### (c) TIG 2009
### Pick a Point inside a 2D loop of Edges
### It makes a new Face in 2D, always facing up
### If the Face exists then it's given the default color and it's 
### flipped upwards if necessary - a quick way of removing any 
### textures or hatching, or simply fixing upside down faces.
### Pick another tool to end or press <enter>.
###
### v2 Reworked 20090920
### v2.5 Clipping fixed with Tool.getExtents 20090921
### v4.0 DeBabelized. Faces inside Groups fixed 20091122
### v4.1 db arguments parenthesized (). 20100927
###
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
class FacemakerTool2D

def db(string)
  dir=File.dirname(__FILE__) ### adjust folder to suit tool
  toolname="2Dtools"  ### adjust xxx to suit tool...
  locale=Sketchup.get_locale.upcase
  path=dir+"/"+toolname+locale+".lingvo"
  if not File.exist?(path)
     return string
  else
    deBabelizer(string,path)
  end 
end#def

def getExtents
    bbox=Sketchup.active_model.bounds
    bbox.add(@ip.position)if @ip and @ip.valid?
    bbox.add(@ip1.position)if @ip1 and @ip1.valid?
    bbox.add(@ip2.position)if @ip2 and @ip2.valid?
    bbox.add(@ip3.position)if @ip3 and @ip3.valid?
    return bbox
end

def initialize
    @ip1 = nil
    @xdown = 0
    @ydown = 0
end

def activate
    ### get/set reference plane 'z'
    @z=Sketchup.active_model.get_attribute("2Dtools","z",nil)
    Sketchup.active_model.set_attribute("2Dtools","z",0.0.mm)if not @z
    @z=Sketchup.active_model.get_attribute("2Dtools","z",nil)
    @zmsg=" [ Z="+@z.to_s+" ] "
    ###
    @cursor=nil
    curpath=File.dirname(__FILE__)+"/Icons/2Dfacemaker_cursor.png"
    @cursor=UI::create_cursor(curpath,0,24)if curpath
    @ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    @exface=nil ### existing face checker
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn and view
    Sketchup::set_status_text("",SB_PROMPT)
    Sketchup::set_status_text("",SB_VCB_LABEL)
    Sketchup::set_status_text("",SB_VCB_VALUE)
end

def onSetCursor()
    UI::set_cursor(@cursor)if @cursor
end

def onReturn(view)
    view.invalidate
    Sketchup::set_status_text("",SB_PROMPT)
    Sketchup.send_action("selectSelectionTool:")
end

def onMouseMove(flags, x, y, view)
    Sketchup::set_status_text((db("2D Face Maker: "))+@zmsg+(db(" Pick Point in Loop to Face, or on Face to flip up & material=<default> , End=<Enter>")), SB_PROMPT)
    Sketchup::set_status_text("",SB_VCB_LABEL)
    @ip.pick view, x, y
    if @ip != @ip1
        view.invalidate if @ip.display? or @ip1.display?
        @ip1.copy! @ip
        view.tooltip = @ip1.tooltip
    end
end

def onLButtonDown(flags, x, y, view)
    @ip1.pick view, x, y
    if @ip1.valid?
        p1=@ip1.position;p1.z=@z
        Sketchup::set_status_text((db("2D Face Maker: "))+@zmsg+(db(" Pick Point in Loop to Face, or on Face to flip up & material=<default> , End=<Enter>")), SB_PROMPT)
        @xdown = x
        @ydown = y
    end
    ### check if extg face
    ph=view.pick_helper
    ph.do_pick(x,y)
    picked_face=ph.picked_face
    @exface=nil
    @exface=picked_face if picked_face and picked_face.bounds.min.z==picked_face.bounds.max.z and picked_face.bounds.min.z==@z
    ###
    self.create_geometry(p1, view)
end

def onLButtonUp(flags, x, y, view)
    Sketchup::set_status_text((db("2D Face Maker: "))+@zmsg+(db(" Pick Point in Loop to Face, or on Face to flip up & material=<default> , End=<Enter>")), SB_PROMPT)
end

def onKeyDown(key, repeat, flags, view)
  ### nothing
end

def onKeyUp(key, repeat, flags, view)
  ### nothing
end

def enableVCB?
   return false
end

def onUserText(text, view)
 ### nothing    
end

def draw(view)
   ### nothing
end

def resume(view)
  view.invalidate
end

def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset(view)
end

def reset(view)
    Sketchup::set_status_text((db("2D Face Maker: "))+@zmsg+(db(" Pick Point in Loop to Face, or on Face to flip up & material=<default> , End=<Enter>")), SB_PROMPT)
    Sketchup::set_status_text("",SB_VCB_LABEL)
    @ip1.clear
    if view 
      view.tooltip = nil
      view.invalidate if @drawn
    end
    @drawn = false
end


def create_geometry(pt, view)
    pt.z=@z
    view.model.start_operation(db("2D Face Maker"))
    ray=[pt,Geom::Vector3d.new(1,0,0)]
    raytest=Sketchup.active_model.raytest(ray)
    edge1=nil
    edge1=raytest[1].last if raytest and raytest[1].last.class==Sketchup::Edge
    return if not edge1
    ray=[pt,Geom::Vector3d.new(-1,0,0)]
    raytest=Sketchup.active_model.raytest(ray)
    edge2=nil
    edge2=raytest[1].last if raytest and raytest[1].last.class==Sketchup::Edge
    return if not edge2
    ###
    exfaces=[]
    edge1.parent.entities.each{|e|exfaces<<e if e.class==Sketchup::Face}
    edge2s=[]
    edge2.all_connected.each{|e|edge2s<<e if e.class==Sketchup::Edge}
    edge2s.each{|e|e.find_faces}
    edges=[]
    edge1.all_connected.each{|e|edges<<e if e.class==Sketchup::Edge}
    edges.each{|e|e.find_faces}
    nowfaces=[]
    edge1.parent.entities.each{|e|nowfaces<<e if e.class==Sketchup::Face and not exfaces.include?(e)}
    nowfaces.each{|face|
      face.erase! if not face.outer_loop.edges.include?(edge1)
      face.erase! if face.valid? and not face.classify_point(pt)==1
      face.reverse! if face.valid? and face.normal==[0,0,-1]
      ###puts face.normal
      face.material=nil if face.valid?
    }
    ### if not made a face then try another ray vector and see if that works
    validface=false
    nowfaces.each{|face|
      if face.valid?
        validface=true; break
      end#if
    }
    if not validface ###########################
      ray=[pt,Geom::Vector3d.new(-1,0,0)]
      raytest=Sketchup.active_model.raytest(ray)
     edge1=nil
    edge1=raytest[1].last if raytest and raytest[1].last.class==Sketchup::Edge
    return if not edge1
    ray=[pt,Geom::Vector3d.new(1,0,0)]
    raytest=Sketchup.active_model.raytest(ray)
    edge2=nil
    edge2=raytest[1].last if raytest and raytest[1].last.class==Sketchup::Edge
    return if not edge2
    ###
    exfaces=[]
    edge1.parent.entities.each{|e|exfaces<<e if e.class==Sketchup::Face}
    edge2s=[]
    edge2.all_connected.each{|e|edge2s<<e if e.class==Sketchup::Edge}
    edge2s.each{|e|e.find_faces}
    edges=[]
    edge1.all_connected.each{|e|edges<<e if e.class==Sketchup::Edge}
    edges.each{|e|e.find_faces}
    nowfaces=[]
    edge1.parent.entities.each{|e|nowfaces<<e if e.class==Sketchup::Face and not exfaces.include?(e)}
    nowfaces.each{|face|
      face.erase! if not face.outer_loop.edges.include?(edge1)
      face.erase! if face.valid? and not face.classify_point(pt)==1
      face.reverse! if face.valid? and face.normal==[0,0,-1]
      face.material=nil if face.valid?
    }
    end#if
    #####################
    ###
    if @exface
      @exface.material=nil
      @exface.reverse! if @exface.normal.z<0
    end#if
    ###
    view.invalidate
    view.model.commit_operation
    reset(view)
end

def draw_geometry(points, view)
  ### nothing
end

end # class

#-----------------------------------------------------------------------------
# shortcut
def facemakertool2D
    Sketchup.active_model.select_tool FacemakerTool2D.new
end
######-----------------------------------------------

