### (c) TIG 2009-2012
### Makes a 2D Radiused Fillet or Chamfer between 2 picked Edges
### 'Alt' key toggles between Fillet and Chamfer
### It extends each picked Edge to suit Arc / Chamfer if it is short.
### It trims each Edge to Arc / Chamfer if Edge has only one / no Faces.
### At any time you can type in a new Fillet Radius - e.g. 200
### or new Chamfer - e.g. 100 OR 100:200 for an uneven chamfer.
### Note how Chamfer is measured in from intersection of the two lines.
### The Tool acts like Trim/Extend if Fillet Radius=0 or Chamfer=0 
### [which are the default settings].
### You can also change the number of Segments in a radiused fillet by 
### typing NNs in the VCB - e.g. 6s for 6 segments: the default is 18.
### The Fillet Radius / Chamfer settings are remembered across sessions 
### for that Model.
### The first starting tooltip is 'Line1' - the Edge picked ('Line1') 
### is highlighted.
### The tooltip changes to 'Line2' when you are over a pickable Edge.
### Picking two non-intersecting Edges, or re-picking Line1 as Line2 
### etc makes a beep and reports an error in Ruby Console.
### To end pick another tool or press <enter>.
###
### v2 Reworked 20090920
### v2.5 Clipping fixed with Tool.getExtents 20090921
### v4.0 DeBabelized. 20091122
### v4.1 db arguments parenthesized (). 20100927
### v4.2 Segments range limited <=2 <=999. 20120228
### v4.3 Side-stepped splats by ill-mannered 3rd parties' EntitiesObservers.
###
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------
class FilletTool2D

def db(string)
  dir=File.dirname(__FILE__) ### adjust folder to suit tool
  toolname="2Dtools"  ### adjust xxx to suit tool...
  locale=Sketchup.get_locale.upcase
  path=dir+"/"+toolname+locale+".lingvo"
  if not File.exist?(path)
     return string
  else
    deBabelizer(string,path)
  end 
end#def

def getExtents
    bbox=Sketchup.active_model.bounds
    bbox.add(@ip.position)if @ip and @ip.valid?
    bbox.add(@ip1.position)if @ip1 and @ip1.valid?
    bbox.add(@ip2.position)if @ip2 and @ip2.valid?
    bbox.add(@ip3.position)if @ip3 and @ip3.valid?
    return bbox
end

def initialize
    @ip = nil
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
    @chamfers=[0.0,0.0] if not @chamfers
    @radius=100.0.mm if not @radius
    @segs=18 if not @segs
end

def activate
    ### get/set reference plane 'z'
    @z=Sketchup.active_model.get_attribute("2Dtools","z",nil)
    Sketchup.active_model.set_attribute("2Dtools","z",0.0.mm)if not @z
    @z=Sketchup.active_model.get_attribute("2Dtools","z",nil)
    ###
    @cursor=nil
    curpath=File.dirname(__FILE__)+"/Icons/2Dfillet_cursor.png"
    @cursor=UI::create_cursor(curpath,0,24)if curpath
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @alt=false
    ### get/set radius/chamfer
    @radius=Sketchup.active_model.get_attribute("2Dtools","fillet_radius",0.0.mm)
    @chamfers=Sketchup.active_model.get_attribute("2Dtools","fillet_chamfers",[0.mm,0.mm])
    @segs=Sketchup.active_model.get_attribute("2Dtools","fillet_segments",18)
    @drawn = false
    @state=0
    @edge1=nil
    @edge2=nil
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn and view
    Sketchup::set_status_text("",SB_PROMPT)
    Sketchup::set_status_text("",SB_VCB_LABEL)
    Sketchup::set_status_text("",SB_VCB_VALUE)
end

def onSetCursor()
    UI::set_cursor(@cursor)if @cursor
end

def onReturn(view)
    view.invalidate
    Sketchup::set_status_text("",SB_PROMPT)
    Sketchup.send_action("selectSelectionTool:")
end

def onMouseMove(flags, x, y, view)
  if @state==0
    if @alt ### chamfer
      Sketchup::set_status_text((db("2D Chamfer: Pick First Line / Enter Chamfer(s) / 'Alt' for Fillet...")), SB_PROMPT)
      Sketchup.set_status_text((db("Chamfer(s)")), SB_VCB_LABEL)
      if @chamfers[0]==@chamfers[1]
        Sketchup.set_status_text(@chamfers[0].to_s,SB_VCB_VALUE)
      else
        Sketchup.set_status_text(@chamfers[0].to_s+":"+@chamfers[1].to_s,SB_VCB_VALUE)
      end#if
    else ### fillet
      Sketchup::set_status_text((db("2D Fillet: Pick First Line / Enter Fillet Radius or NNs Segments / 'Alt' for Chamfer...")), SB_PROMPT)
      Sketchup.set_status_text((db("Fillet Radius")), SB_VCB_LABEL)
      Sketchup.set_status_text(@radius.to_s,SB_VCB_VALUE)
    end#if
    @ip.pick view, x, y
    if @ip != @ip1
        view.invalidate if @ip.display? or @ip1.display?
        @ip1.copy! @ip
        view.tooltip = db("Line 1")
    end#if
  else ###state==1
    if @alt
      Sketchup::set_status_text((db("2D Chamfer: Pick Second Line / Enter Chamfer(s) / 'Alt' for Fillet...")), SB_PROMPT)
      Sketchup.set_status_text((db("Chamfer(s)")), SB_VCB_LABEL)
      if @chamfers[0]==@chamfers[1]
        Sketchup.set_status_text(@chamfers[0].to_s,SB_VCB_VALUE)
      else
        Sketchup.set_status_text(@chamfers[0].to_s+":"+@chamfers[1].to_s,SB_VCB_VALUE)
      end#if
    else
      Sketchup::set_status_text((db("2D Fillet: Pick Second Line / Enter Fillet Radius or NNs Segments / 'Alt' for Chamfer...")), SB_PROMPT)
      Sketchup.set_status_text((db("Fillet Radius")), SB_VCB_LABEL)
      Sketchup.set_status_text(@radius.to_s,SB_VCB_VALUE)
    end#if
    @ip.pick view, x, y
    if @ip != @ip2
        view.invalidate if @ip.display? or @ip2.display?
        @ip2.copy! @ip
        view.tooltip = db("Line 2")
    end#if
  end#if
end

def onLButtonDown(flags, x, y, view)
  if @state==0
    @ip1.pick view, x, y
    if @ip1.valid?
        p1=@ip1.position;p1.z=@z
        Sketchup::set_status_text((db("2D Chamfer: Pick Second Line / Enter Chamfer(s) / 'Alt' for Fillet...")), SB_PROMPT)if @alt
        Sketchup::set_status_text((db("2D Fillet: Pick Second Line / Enter Fillet Radius or NNs Segments / 'Alt' for Chamfer...")), SB_PROMPT)if not @alt
        @xdown = x
        @ydown = y
    end
    ph=view.pick_helper
    ph.do_pick(x,y)
    picked_edge=ph.picked_edge
    @edge1=nil
    @edge1=picked_edge if picked_edge
    if @edge1
      view.model.selection.clear
      view.model.selection.add(@edge1)
      @state=1
    end#if
  else ### state==2
    if @ip2.valid?
      @xdown = x
      @ydown = y
    end#if
    @ip2.pick view, x, y
    ph=view.pick_helper
    ph.do_pick(x,y)
    picked_edge=ph.picked_edge
    @edge2=nil
    @edge2=picked_edge if picked_edge
    if @edge2
      crosspoint=nil
      crosspoint=Geom.intersect_line_line(@edge1.line,@edge2.line)if @edge1.valid? and @edge2.valid?
      if not crosspoint
        UI.beep
        puts db("No intersection!")
        @edge2=nil
        @ip2.clear
      end#if
      if @edge1 and @edge2 and @edge1.valid? and @edge2.valid? and @edge2!=@edge1
        self.create_geometry(view)
      elsif @edge1 and @edge2 and @edge1.valid? and @edge2.valid? and @edge2==@edge1
        UI.beep
        puts db("Line 2 CANNOT == Line 1 !")
        @edge2=nil
        @ip2.clear
      elsif @edge1 and @edge2 and (not @edge1.valid? or not @edge2.valid?)
        UI.beep
        puts db("Picked Edge(s) Error !")
        @edge2=nil
        @ip2.clear
      end#if
    end#if
  end#if
end

def onLButtonUp(flags, x, y, view)
    ###
end

def onKeyDown(key, repeat, flags, view)
  if key==VK_ALT  ### ALT toggles fillet / chamfer
      if @alt
        @alt=false
        Sketchup.set_status_text((db("Fillet Radius")), SB_VCB_LABEL)
        Sketchup::set_status_text((db("2D Fillet: Pick First Line / Enter Fillet Radius or NNs Segments / 'Alt' for Chamfer...")), SB_PROMPT)if @state==0
        Sketchup::set_status_text((db("2D Fillet: Pick Second Line / Enter Fillet Radius or NNs Segments / 'Alt' for Chamfer...")), SB_PROMPT)if @state==1
        Sketchup.set_status_text(@radius.to_s,SB_VCB_VALUE)
        self.onSetCursor()
      else
        @alt=true
        Sketchup.set_status_text((db("Chamfer(s)")), SB_VCB_LABEL)
        Sketchup::set_status_text((db("2D Chamfer: Pick First Line / Enter Chamfer(s) / 'Alt' for Fillet...")), SB_PROMPT)if @state==0
        Sketchup::set_status_text((db("2D Chamfer: Pick Second Line / Enter Chamfer(s) / 'Alt' for Fillet...")), SB_PROMPT)if @state==1
        if @chamfers[0]==@chamfers[1]
          Sketchup.set_status_text(@chamfers[0].to_s,SB_VCB_VALUE)
        else
          Sketchup.set_status_text(@chamfers[0].to_s+":"+@chamfers[1].to_s,SB_VCB_VALUE)
        end#if
        self.onSetCursor()
      end#if
    end#if
end

def onKeyUp(key, repeat, flags, view)
  ### nothing
end

def enableVCB?
   return true
end

def onUserText(text, view)
  if @alt ### =chamfered
    Sketchup.set_status_text((db("Chamfer(s)")), SB_VCB_LABEL)
    @chamfers=text.split(":")
    if @chamfers.last!=text ###e.g. 100:200
      if not @chamfers[0]
        @chamfers=[0.0,0.0]
      elsif @chamfers[0]
        @chamfers[0]=@chamfers[0].to_l
        if @chamfers[1]
          @chamfers[1]=@chamfers[1].to_l
        else
          @chamfers[1]=0.0
        end#if
      end#if
    else
      @chamfers=[text.to_l ,text.to_l]### both ==
    end#if
    Sketchup.active_model.set_attribute("2Dtools","fillet_chamfers",@chamfers)
    if @chamfers[0]==@chamfers[1]
      Sketchup.set_status_text(@chamfers[0].to_s,SB_VCB_VALUE)
    else
      Sketchup.set_status_text(@chamfers[0].to_s+":"+@chamfers[1].to_s,SB_VCB_VALUE)
    end#if
  else ### it's a radius fillet
    Sketchup.set_status_text((db("Fillet Radius")), SB_VCB_LABEL)
    ### check if ends with 's' and set segs
    if text.split("").last=="s" ### e.g. 12s
      @lastseg = @segs
      @segs = text.to_i 
      if @segs < 2 or @segs > 999
        UI.beep 
        @segs = @lastseg 
      end
      Sketchup.set_status_text(@segs.to_s,SB_VCB_VALUE)
    else ### it's a radius
      @radius=text.to_l if text
      Sketchup.set_status_text(@radius.to_s,SB_VCB_VALUE)
      Sketchup.active_model.set_attribute("2Dtools","fillet_radius",@radius)
      Sketchup.active_model.set_attribute("2Dtools","fillet_segments",@segs)
    end#if
  end#if
  view.invalidate
end

def draw(view)
   ### nothing
end

def onCancel(flag, view)
    view.invalidate if @drawn
    view.model.selection.clear
    self.reset(view)
end

def reset(view)
    if @alt
      Sketchup::set_status_text((db("2D Chamfer: Pick First Line / Enter Chamfer(s) / 'Alt' for Fillet...")), SB_PROMPT)
    else
      Sketchup::set_status_text((db("2D Fillet: Pick First Line / Enter Fillet Radius or NNs Segments / 'Alt' for Chamfer...")), SB_PROMPT)
    end#if
    @ip.clear
    @ip1.clear
    @ip2.clear
    if view 
      view.tooltip = nil
      view.invalidate if @drawn
    end
    @drawn = false
    @state=0
    @edge1=nil
    @edge2=nil
end

def create_geometry(view)
    model=view.model
    model.selection.clear
    model.start_operation(db("2D Chamfer"))if @alt
    model.start_operation(db("2D Fillet"))if not @alt
    ### using @edge1/@edge2 and @ip1/@ip2
    ents=model.active_entities
    ### check intersection
    crosspoint=nil
    crosspoint=Geom.intersect_line_line(@edge1.line,@edge2.line)if @edge1.valid? and @edge2.valid?
    if not crosspoint
      UI.beep
      puts db("No intersection!")
      return nil
    end#if
    if @alt ### ==chamfer
        e1=@edge1.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[0].position
        ee1=@edge1.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[1].position
        p1=e1.offset((ee1-e1),(e1.distance(crosspoint))-@chamfers[0])
        e2=@edge2.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[0].position
        ee2=@edge2.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[1].position
        p2=e2.offset((ee2-e2),(e2.distance(crosspoint))-@chamfers[1])
        @edge1.erase! if @edge1.valid? and not @edge1.faces[0]
        @edge2.erase! if @edge2.valid? and not @edge2.faces[0]
        ed1=ents.add_line(p1,e1)
        ed2=ents.add_line(p2,e2)
        ed1x=ents.add_line(p1,ee1)if e1.distance(p1)<e1.distance(ee1)
        ed2x=ents.add_line(p2,ee2)if e2.distance(p2)<e2.distance(ee2)
        ### trim edges if got only one face and its 'common'
      if @chamfers[0]!=0
        ce=ents.add_line(p1,p2)
        ed1xfaces=[];ed1xfaces=ed1x.faces if ed1x
        ed2xfaces=[];ed2xfaces=ed2x.faces if ed2x
        crosspt=false;crosspt=true if e1==e2 or ee1==ee2 or e1==ee2 or ee1==e2
        togo=[]
        togo<<ed1x if ed1x and ed1x.valid? and ed1xfaces[0] and not ed1xfaces[1] and ed2xfaces.include?(ed1xfaces[0])and crosspt
        togo<<ed2x if ed2x and ed2x.valid? and ed2xfaces[0] and not ed2xfaces[1] and ed1xfaces.include?(ed2xfaces[0])and crosspt
        togo<<ed1x if ed1x and not ed1x.faces[0]
        togo<<ed2x if ed2x and not ed2x.faces[0]
        togo.uniq!
        togo.each{|e|e.erase! if e.valid?}
      end#if
      ###
    else ### ==radiused fillet
      ###
        e1=@edge1.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[0].position
        ee1=@edge1.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[1].position
        v1=e1-crosspoint
        e2=@edge2.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[0].position
        ee2=@edge2.vertices.sort_by{|i|i.position.distance crosspoint}.reverse[1].position
        v2=e2-crosspoint
        v3=Geom::Vector3d.linear_combination(0.5,v1.normalize,0.5,v2.normalize)
        len5=(@radius/Math.sin(v2.angle_between(v1)/2))
        len4=(@radius/Math.tan(v2.angle_between(v1)/2))
        v1.length=len4
        v2.length=len4
        v3.length=len5
        p1=crosspoint.offset(v1)
        p2=crosspoint.offset(v2)
        p3=crosspoint.offset(v3)
        @edge1.erase! if not @edge1.faces[0]
        @edge2.erase! if not @edge2.faces[0]
        ed1=ents.add_line(p1,e1)
        ed2=ents.add_line(p2,e2)
        ed1x=ents.add_line(p1,ee1)if e1.distance(p1)<e1.distance(ee1)
        ed2x=ents.add_line(p2,ee2)if e2.distance(p2)<e2.distance(ee2)
      ### make arc and trim edges if got only one face and its 'common'
      if @radius!=0
        center=p3
        xaxis=v3.reverse
        normal=v1*v2
        radius=@radius
        sangle= +(v3.angle_between(p3-p1))
        eangle= -(v3.angle_between(p3-p1))
        segs=@segs
        arcedges=ents.add_arc(center,xaxis,normal,radius,sangle,eangle,segs)
        ### arc+all_connected must be grouped/copied/exploded 
        ### to force edges into arc for this session
        arcbits=[arcedges[0]];arc_con=arcedges[0].all_connected
        arc_con.each{|e|arcbits<<e if e.valid?}
        gp1=ents.add_group(arcbits)
        gp2=ents.add_instance(gp1.entities.parent, gp1.transformation)
		gp2.explode
		gp1.entities.clear!
        ### now tidy loose edges...
        ed1x=ents.add_line(p1,ee1)if e1.distance(p1)<e1.distance(ee1)
        ed2x=ents.add_line(p2,ee2)if e2.distance(p2)<e2.distance(ee2)
        ed1xfaces=[];ed1xfaces=ed1x.faces if ed1x and ed1x.valid?
        ed2xfaces=[];ed2xfaces=ed2x.faces if ed2x and ed2x.valid?
        crosspt=false;crosspt=true if e1==e2 or ee1==ee2 or e1==ee2 or ee1==e2
        togo=[]
        togo<<ed1x if ed1x and ed1x.valid? and ed1xfaces[0] and ed1xfaces[0].valid? and not ed1xfaces[1] and ed2xfaces.include?(ed1xfaces[0])and crosspt
        togo<<ed2x if ed2x and ed2x.valid? and ed2xfaces[0] and ed2xfaces[0].valid? and not ed2xfaces[1] and ed1xfaces.include?(ed2xfaces[0])and crosspt
        togo<<ed1x if ed1x and ed1x.valid? and not ed1x.faces[0]
        togo<<ed2x if ed2x and ed2x.valid? and not ed2x.faces[0]
        togo.uniq!
        togo.each{|e|e.erase! if e.valid?}
        ents.add_cpoint(p3)### add arc centerpoint
      end#if radius!=0
    end#if
    ###
    view.invalidate
    view.model.commit_operation
    reset(view)
end

def draw_geometry(points, view)
  ### nothing
end

end # class

#-----------------------------------------------------------------------------
# shortcut
def fillettool2D
    Sketchup.active_model.select_tool FilletTool2D.new
end
######-----------------------------------------------

